==> vdr-plugin-webvideo_0.2.2.orig.tar.gz: webvideo-0.2.2/src/vdr-plugin/history.c <==
/*
 * history.c: Web video plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include "history.h"
#include "menu.h"
#include "common.h"

// --- cHistoryObject -----------------------------------------------------

cHistoryObject::cHistoryObject(const cString &xml, int ID) {
  osdxml = strdup(xml);
  id = ID;
  selected = 0;
}

cHistoryObject::~cHistoryObject() {
  if (osdxml)
    free(osdxml);

  for (int i=0; i < editData.Size(); i++)
    delete editData[i];
}

cQueryData *cHistoryObject::GetEditItem(const char *controlName) {
  debug("GetEditItem %s", controlName);

  for (int i=0; i < editData.Size(); i++) {
    if (strcmp(editData[i]->GetName(), controlName) == 0) {

      debug("GetEditItem returning %d:th object", i);

      return editData[i];
    }
  }

  return NULL;
}

int cHistoryObject::QuerySize() const {
  return editData.Size();
}

char *cHistoryObject::GetQueryFragment(int i) const {
  if (i < 0 && i >= editData.Size())
    return NULL;
  else
    return editData[i]->GetQueryFragment();
}

cTextFieldData *cHistoryObject::GetTextFieldData(const char *controlName) {
  cQueryData *edititem = GetEditItem(controlName);
  cTextFieldData *tfdata = dynamic_cast<cTextFieldData *>(edititem);

  debug("GetTextFieldData: tfdata = %p", tfdata);

  if (!tfdata) {

    debug("GetTextFieldData: constructing new cTextFieldData");

    tfdata = new cTextFieldData(controlName, 256);

    debug("appending tfdata");

    editData.Append(tfdata);
  }

  return tfdata;
}

cItemListData *cHistoryObject::GetItemListData(const char *controlName,
                                               cStringList &items,
                                               cStringList &values) {
  int n;
  char **itemtable, **itemvaluetable;
  cQueryData *edititem = GetEditItem(controlName);
  cItemListData *ildata = dynamic_cast<cItemListData *>(edititem);

  debug("GetItemListData: ildata = %p", ildata);

  if (!ildata) {
    debug("GetItemListData: constructing new cItemListData");

    n = min(items.Size(), values.Size());
    
    debug("GetItemListData: number = %d", n);

    itemtable = (char **)malloc(n*sizeof(char *));
    itemvaluetable = (char **)malloc(n*sizeof(char *));

    debug("tables %p %p", itemtable, itemvaluetable);

    for (int i=0; i<n; i++) {
      itemtable[i] = strdup(csc.Convert(items[i]));
      itemvaluetable[i] = strdup(values[i]);
    }
    
    ildata = new cItemListData(controlName,
                               itemtable,
                               itemvaluetable,
                               n);

    debug("appending ildata");

    editData.Append(ildata);
  }

  return ildata;
}

// --- cHistory ------------------------------------------------------------

cHistory::cHistory() {
  current = NULL;
}

void cHistory::Clear() {
  current = NULL;
  cList<cHistoryObject>::Clear();
}

void cHistory::TruncateAndAdd(cHistoryObject *page) {
  cHistoryObject *last = Last();
  while ((last) && (last != current)) {
    Del(last);
    last = Last();
  }

  Add(page);
  current = Last();
}

void cHistory::Reset() {
  current = NULL;
}

cHistoryObject *cHistory::Current() {
  return current;
}

cHistoryObject *cHistory::Home() {
  current = First();
  return current;
}

cHistoryObject *cHistory::Back() {
  if (current)
    current = Prev(current);
  return current;
}

cHistoryObject *cHistory::Forward() {
  cHistoryObject *next;
  if (current) {
    next = Next(current);
    if (next)
      current = next;
  } else {
    current = First();
  }
  return current;
}

==> vdr-plugin-webvideo_0.3.2.orig.tar.gz: webvideo-0.3.2/src/vdr-plugin/history.c <==
/*
 * history.c: Web video plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <string.h>
#include "history.h"
#include "menu.h"

// --- cHistoryObject -----------------------------------------------------

cHistoryObject::cHistoryObject(const char *xml, const char *ref, int ID) {
  osdxml = strdup(xml);
  reference = strdup(ref);
  id = ID;
  selected = 0;
}

cHistoryObject::~cHistoryObject() {
  if (osdxml)
    free(osdxml);
  if (reference)
    free(reference);

  for (int i=0; i < editData.Size(); i++)
    delete editData[i];
}

cQueryData *cHistoryObject::GetEditItem(const char *controlName) {
  for (int i=0; i < editData.Size(); i++) {
    if (strcmp(editData[i]->GetName(), controlName) == 0) {
      return editData[i];
    }
  }

  return NULL;
}

int cHistoryObject::QuerySize() const {
  return editData.Size();
}

char *cHistoryObject::GetQueryFragment(int i) const {
  if (i < 0 && i >= editData.Size())
    return NULL;
  else
    return editData[i]->GetQueryFragment();
}

cTextFieldData *cHistoryObject::GetTextFieldData(const char *controlName) {
  cQueryData *edititem = GetEditItem(controlName);
  cTextFieldData *tfdata = dynamic_cast<cTextFieldData *>(edititem);

  if (!tfdata) {
    tfdata = new cTextFieldData(controlName, 256);
    editData.Append(tfdata);
  }

  return tfdata;
}

cItemListData *cHistoryObject::GetItemListData(const char *controlName,
                                               cStringList &items,
                                               cStringList &values) {
  int n;
  char **itemtable, **itemvaluetable;
  cQueryData *edititem = GetEditItem(controlName);
  cItemListData *ildata = dynamic_cast<cItemListData *>(edititem);

  if (!ildata) {
    n = min(items.Size(), values.Size());
    itemtable = (char **)malloc(n*sizeof(char *));
    itemvaluetable = (char **)malloc(n*sizeof(char *));

    for (int i=0; i<n; i++) {
      itemtable[i] = strdup(csc.Convert(items[i]));
      itemvaluetable[i] = strdup(values[i]);
    }
    
    ildata = new cItemListData(controlName,
                               itemtable,
                               itemvaluetable,
                               n);

    editData.Append(ildata);
  }

  return ildata;
}

// --- cHistory ------------------------------------------------------------

cHistory::cHistory() {
  current = NULL;
}

void cHistory::Clear() {
  current = NULL;
  cList<cHistoryObject>::Clear();
}

void cHistory::TruncateAndAdd(cHistoryObject *page) {
  cHistoryObject *last = Last();
  while ((last) && (last != current)) {
    Del(last);
    last = Last();
  }

  Add(page);
  current = Last();
}

void cHistory::Reset() {
  current = NULL;
}

cHistoryObject *cHistory::Current() {
  return current;
}

cHistoryObject *cHistory::Home() {
  current = First();
  return current;
}

cHistoryObject *cHistory::Back() {
  if (current)
    current = Prev(current);
  return current;
}

cHistoryObject *cHistory::Forward() {
  cHistoryObject *next;
  if (current) {
    next = Next(current);
    if (next)
      current = next;
  } else {
    current = First();
  }
  return current;
}
